@@ -8,7 +8,7 @@ from __future__ import print_function, absolute_import
 import string
 
 from ..six import StringIO
-from . import types
+from . import types, _utils
 from ._utils import _StrCaching, _StringReferenceCaching
 
 
@@ -17,42 +17,24 @@ _VALID_CHARS = (frozenset(map(ord, string.ascii_letters)) |
                 frozenset('._-$'))
 
 
-def _escape_string(text):
-    buf = []
-    for ch in text:
-        if ch in _VALID_CHARS:
-            buf.append(chr(ch))
-        else:
-            ashex = hex(ch)[2:]
-            if len(ashex) == 1:
-                ashex = '0' + ashex
-            buf.append('\\' + ashex)
+def _escape_string(text, _map={}):
+    """
+    Escape the given bytestring for safe use as a LLVM array constant.
+    """
+    assert isinstance(text, (bytes, bytearray))
+
+    if not _map:
+        for ch in range(256):
+            if ch in _VALID_CHARS:
+                _map[ch] = chr(ch)
+            else:
+                _map[ch] = '\\%02x' % ch
+
+    buf = [_map[ch] for ch in text]
     return ''.join(buf)
 
 
-class _Undefined(object):
-    pass
-
-Undefined = _Undefined()
-
-
-class ConstOp(object):
-    """
-    A simple value-like object representing the result of a constant operation.
-    """
-
-    def __init__(self, typ, op):
-        assert isinstance(typ, types.Type)
-        self.type = typ
-        self.op = op
-
-    def __str__(self):
-        return self.op
-
-    get_reference = __str__
-
-
-class ConstOpMixin(object):
+class _ConstOpMixin(object):
     """
     A mixin defining constant operations, for use in constant-like classes.
     """
@@ -65,25 +47,31 @@ class ConstOpMixin(object):
             return self
         op = "bitcast ({0} {1} to {2})".format(self.type, self.get_reference(),
                                                typ)
-        return ConstOp(typ, op)
+        return FormattedConstant(typ, op)
 
     def inttoptr(self, typ):
         """
         Cast this integer constant to the given pointer type.
         """
-        assert isinstance(self.type, types.IntType)
-        assert isinstance(typ, types.PointerType)
+        if not isinstance(self.type, types.IntType):
+            raise TypeError("can only call inttoptr() on integer constants, not '%s'"
+                            % (self.type,))
+        if not isinstance(typ, types.PointerType):
+            raise TypeError("can only inttoptr() to pointer type, not '%s'"
+                            % (typ,))
+
         op = "inttoptr ({0} {1} to {2})".format(self.type,
                                                 self.get_reference(),
                                                 typ)
-        return ConstOp(typ, op)
+        return FormattedConstant(typ, op)
 
     def gep(self, indices):
         """
         Call getelementptr on this pointer constant.
         """
         if not isinstance(self.type, types.PointerType):
-            raise TypeError("cannot only call gep() on pointer constants")
+            raise TypeError("can only call gep() on pointer constants, not '%s'"
+                            % (self.type,))
 
         outtype = self.type
         for i in indices:
@@ -95,10 +83,27 @@ class ConstOpMixin(object):
         op = "getelementptr ({0}, {1} {2}, {3})".format(
             self.type.pointee, self.type,
             self.get_reference(), ', '.join(strindices))
-        return ConstOp(outtype.as_pointer(), op)
+        return FormattedConstant(outtype.as_pointer(), op)
 
 
-class Constant(_StrCaching, _StringReferenceCaching, ConstOpMixin):
+class Value(object):
+    """
+    The base class for all values.
+    """
+
+    def __repr__(self):
+        return "<ir.%s type='%s' ...>" % (self.__class__.__name__, self.type,)
+
+
+class _Undefined(object):
+    """
+    'undef': a value for undefined values.
+    """
+
+Undefined = _Undefined()
+
+
+class Constant(_StrCaching, _StringReferenceCaching, _ConstOpMixin, Value):
     """
     A constant LLVM value.
     """
@@ -107,23 +112,26 @@ class Constant(_StrCaching, _StringReferenceCaching, ConstOpMixin):
         assert isinstance(typ, types.Type)
         assert not isinstance(typ, types.VoidType)
         self.type = typ
+        if isinstance(constant, (list, tuple)):
+            # Recursively wrap aggregate constants
+            constant = typ.wrap_constant_value(constant)
         self.constant = constant
 
     def _to_string(self):
         return '{0} {1}'.format(self.type, self.get_reference())
 
     def _get_reference(self):
-        if isinstance(self.constant, bytearray):
-            val = 'c"{0}"'.format(_escape_string(self.constant))
-
-        elif self.constant is None:
+        if self.constant is None:
             val = self.type.null
 
         elif self.constant is Undefined:
             val = "undef"
 
+        elif isinstance(self.constant, bytearray):
+            val = 'c"{0}"'.format(_escape_string(self.constant))
+
         else:
-            val = self.type.format_const(self.constant)
+            val = self.type.format_constant(self.constant)
 
         return val
 
@@ -147,19 +155,38 @@ class Constant(_StrCaching, _StringReferenceCaching, ConstOpMixin):
     def __hash__(self):
         return hash(str(self))
 
+    def __repr__(self):
+        return "<ir.Constant type='%s' value=%r>" % (self.type, self.constant)
 
-class Value(_StrCaching, _StringReferenceCaching):
+
+class FormattedConstant(Constant):
+    """
+    A constant with an already formatted IR representation.
+    """
+
+    def __init__(self, typ, constant):
+        assert isinstance(constant, str)
+        Constant.__init__(self, typ, constant)
+
+    def _to_string(self):
+        return self.constant
+
+    def _get_reference(self):
+        return self.constant
+
+
+class NamedValue(_StrCaching, _StringReferenceCaching, Value):
+    """
+    The base class for named values.
+    """
     name_prefix = '%'
     deduplicate_name = True
-    creates_nested_scope = False
 
     def __init__(self, parent, type, name):
         assert parent is not None
         assert isinstance(type, types.Type)
         self.parent = parent
         self.type = type
-        pscope = self.parent.scope
-        self.scope = pscope.get_child() if self.creates_nested_scope else pscope
         self._set_name(name)
 
     def _to_string(self):
@@ -176,7 +203,7 @@ class Value(_StrCaching, _StringReferenceCaching):
         return self._name
 
     def _set_name(self, name):
-        name = self.scope.register(name, deduplicate=self.deduplicate_name)
+        name = self.parent.scope.register(name, deduplicate=self.deduplicate_name)
         self._name = name
 
     name = property(_get_name, _set_name)
@@ -188,6 +215,10 @@ class Value(_StrCaching, _StringReferenceCaching):
             name = name.replace('\\', '\\5c').replace('"', '\\22')
         return '{0}"{1}"'.format(self.name_prefix, name)
 
+    def __repr__(self):
+        return "<ir.%s %r of type '%s'>" % (
+            self.__class__.__name__, self.name, self.type)
+
     @property
     def function_type(self):
         ty = self.type
@@ -199,7 +230,7 @@ class Value(_StrCaching, _StringReferenceCaching):
             raise TypeError("Not a function: {0}".format(self.type))
 
 
-class MetaDataString(Value):
+class MetaDataString(NamedValue):
     """
     A metadata string, i.e. a constant string used as a value in a metadata
     node.
@@ -240,7 +271,7 @@ class NamedMetaData(object):
         self.operands.append(md)
 
 
-class MDValue(Value):
+class MDValue(NamedValue):
     name_prefix = '!'
 
     def __init__(self, parent, values, name):
@@ -257,7 +288,10 @@ class MDValue(Value):
         for op in self.operands:
             typestr = str(op.type)
             if isinstance(op.type, types.MetaData):
-                operands.append(op.get_reference())
+                if isinstance(op, Constant) and op.constant == None:
+                    operands.append("null")
+                else:
+                    operands.append(op.get_reference())
             else:
                 operands.append("{0} {1}".format(op.type, op.get_reference()))
         operands = ', '.join(operands)
@@ -279,7 +313,7 @@ class MDValue(Value):
         return hash(self.operands)
 
 
-class GlobalValue(Value, ConstOpMixin):
+class GlobalValue(NamedValue, _ConstOpMixin):
     """
     A global value.
     """
@@ -363,6 +397,7 @@ class FunctionAttributes(AttributeSet):
 
     def __init__(self):
         self._alignstack = 0
+        self._personality = None
 
     @property
     def alignstack(self):
@@ -373,10 +408,23 @@ class FunctionAttributes(AttributeSet):
         assert val >= 0
         self._alignstack = val
 
+    @property
+    def personality(self):
+        return self._personality
+
+    @personality.setter
+    def personality(self, val):
+        assert val is None or isinstance(val, GlobalValue)
+        self._personality = val
+
     def __repr__(self):
         attrs = sorted(self)
         if self.alignstack:
             attrs.append('alignstack({0:d})'.format(self.alignstack))
+        if self.personality:
+            attrs.append('personality {persty} {persfn}'.format(
+                            persty=self.personality.type,
+                            persfn=self.personality.get_reference()))
         return ' '.join(attrs)
 
 
@@ -384,12 +432,11 @@ class Function(GlobalValue):
     """Represent a LLVM Function but does uses a Module as parent.
     Global Values are stored as a set of dependencies (attribute `depends`).
     """
-    creates_nested_scope = True
-
     def __init__(self, module, ftype, name):
         assert isinstance(ftype, types.Type)
         super(Function, self).__init__(module, ftype.as_pointer(), name=name)
         self.ftype = ftype
+        self.scope = _utils.NameScope()
         self.blocks = []
         self.attributes = FunctionAttributes()
         self.args = tuple([Argument(self, t)
@@ -471,7 +518,7 @@ class ArgumentAttributes(AttributeSet):
                         'sret', 'zeroext'])
 
 
-class _BaseArgument(Value):
+class _BaseArgument(NamedValue):
     def __init__(self, parent, typ, name=''):
         assert isinstance(typ, types.Type)
         super(_BaseArgument, self).__init__(parent, typ, name=name)
@@ -479,7 +526,7 @@ class _BaseArgument(Value):
         self.attributes = ArgumentAttributes()
 
     def __repr__(self):
-        return "<Argument %r of type %s>" % (self.name, self.type)
+        return "<ir.%s %r of type %s>" % (self.__class__.__name__, self.name, self.type)
 
     def add_attribute(self, attr):
         self.attributes.add(attr)
@@ -510,7 +557,7 @@ class ReturnValue(_BaseArgument):
             return str(self.type)
 
 
-class Block(Value):
+class Block(NamedValue):
     """
     A LLVM IR basic block. A basic block is a sequence of
     instructions whose execution always goes from start to end.  That
@@ -521,6 +568,7 @@ class Block(Value):
 
     def __init__(self, parent, name=''):
         super(Block, self).__init__(parent, types.LabelType(), name=name)
+        self.scope = parent.scope
         self.instructions = []
         self.terminator = None
 
@@ -549,7 +597,7 @@ class Block(Value):
                 instr.replace_usage(old, new)
 
 
-class BlockAddress(object):
+class BlockAddress(Value):
     """
     The address of a basic block.
     """
