@@ -4,12 +4,13 @@ import ctypes
 from ctypes import *
 from ctypes.util import find_library
 import gc
+import locale
+import os
+import platform
+import re
 import subprocess
 import sys
 import unittest
-import platform
-import locale
-import re
 
 from llvmlite import six
 from llvmlite import binding as llvm
@@ -139,6 +140,42 @@ class BaseTest(TestCase):
         return target.create_target_machine()
 
 
+class TestDependencies(BaseTest):
+    """
+    Test DLL dependencies are within a certain expected set.
+    """
+
+    @unittest.skipUnless(sys.platform.startswith('linux'), "Linux-specific test")
+    def test_linux(self):
+        lib_path = ffi.lib._name
+        env = os.environ.copy()
+        env['LANG'] = 'C'
+        p = subprocess.Popen(["objdump", "-p", lib_path],
+                             stdout=subprocess.PIPE, env=env)
+        out, _ = p.communicate()
+        self.assertEqual(0, p.returncode)
+        # Parse library dependencies
+        lib_pat = re.compile(r'^([-_a-zA-Z0-9]+)\.so(?:\.\d+)?$')
+        deps = set()
+        for line in out.decode().splitlines():
+            parts = line.split()
+            if parts and parts[0] == 'NEEDED':
+                dep = parts[1]
+                m = lib_pat.match(dep)
+                if len(parts) != 2 or not m:
+                    self.fail("invalid NEEDED line: %r" % (line,))
+                deps.add(m.group(1))
+        # Sanity check that our dependencies were parsed ok
+        if 'libc' not in deps or 'libpthread' not in deps:
+            self.fail("failed parsing dependencies? got %r" % (deps,))
+        # Ensure all dependencies are expected
+        allowed = set(['librt', 'libdl', 'libpthread', 'libz', 'libm',
+                       'libgcc_s', 'libc', 'ld-linux'])
+        for dep in deps:
+            if not dep.startswith('ld-linux-') and dep not in allowed:
+                self.fail("unexpected dependency %r in %r" % (dep, deps))
+
+
 class TestMisc(BaseTest):
     """
     Test miscellaneous functions in llvm.binding.
@@ -757,6 +794,11 @@ class TestTargetData(BaseTest):
 
 class TestTargetMachine(BaseTest):
 
+    def test_add_target_data_pass(self):
+        tm = self.target_machine()
+        pm = llvm.create_module_pass_manager()
+        tm.target_data.add_pass(pm)
+
     def test_add_analysis_passes(self):
         tm = self.target_machine()
         pm = llvm.create_module_pass_manager()
