@@ -108,9 +108,13 @@ class TestFunction(TestBase):
         func.attributes.add("optsize")
         func.attributes.add("alwaysinline")
         func.attributes.alignstack = 16
+        tp_pers = ir.FunctionType(int8, (), var_arg=True)
+        pers = ir.Function(self.module(), tp_pers, '__gxx_personality_v0')
+        func.attributes.personality = pers
         asm = self.descr(func).strip()
         self.assertEqual(asm,
-            "declare %s alwaysinline optsize alignstack(16)" % self.proto)
+            ("declare %s alwaysinline optsize alignstack(16) "
+             "personality i8 (...)* @\"__gxx_personality_v0\"") % self.proto)
 
     def test_function_attributes(self):
         # Now with parameter attributes
@@ -173,6 +177,9 @@ class TestIR(TestBase):
         self.assertEqual(mod.get_global('foo'), foo)
         self.assertEqual(mod.get_global('globdouble'), globdouble)
         self.assertIsNone(mod.get_global('kkk'))
+        # Globals should have a useful repr()
+        self.assertEqual(repr(globdouble),
+                         "<ir.GlobalVariable 'globdouble' of type 'double*'>")
 
     def test_functions_global_values_access(self):
         mod = self.module()
@@ -221,6 +228,12 @@ class TestIR(TestBase):
         self.assertInText("!bar = !{ !0, !0 }", str(mod))
         self.assert_valid_ir(mod)
 
+    def test_metadata_null(self):
+        mod = self.module()
+        md = mod.add_metadata([ir.Constant(ir.PointerType(ir.IntType(32)), None)])
+        self.assertInText("!{ i32* null }", str(mod))
+        self.assert_valid_ir(mod)
+
     def test_inline_assembly(self):
         mod = self.module()
         foo = ir.Function(mod, ir.FunctionType(ir.VoidType(), []), 'foo')
@@ -264,6 +277,14 @@ class TestBlock(TestBase):
                 %"e" = mul i32 %"f", %".2"
             """)
 
+    def test_repr(self):
+        """
+        Blocks should have a useful repr()
+        """
+        func = self.function()
+        block = ir.Block(parent=func, name='start')
+        self.assertEqual(repr(block), "<ir.Block 'start' of type 'label'>")
+
 
 class TestBuildInstructions(TestBase):
     """
@@ -276,11 +297,15 @@ class TestBuildInstructions(TestBase):
         block = self.block(name='my_block')
         builder = ir.IRBuilder(block)
         a, b = builder.function.args[:2]
-        builder.add(a, b, 'res')
+        inst = builder.add(a, b, 'res')
         self.check_block(block, """\
             my_block:
                 %"res" = add i32 %".1", %".2"
             """)
+        # Instructions should have a useful repr()
+        self.assertEqual(repr(inst),
+                         "<ir.Instruction 'res' of type 'i32', opname 'add', "
+                         "operands (<ir.Argument '.1' of type i32>, <ir.Argument '.2' of type i32>)>")
 
     def test_binops(self):
         block = self.block(name='my_block')
@@ -778,9 +803,7 @@ class TestBuildInstructions(TestBase):
     def test_landingpad(self):
         block = self.block(name='my_block')
         builder = ir.IRBuilder(block)
-        tp_pers = ir.FunctionType(int8, (), var_arg=True)
-        pers = ir.Function(builder.function.module, tp_pers, '__gxx_personality_v0')
-        lp = builder.landingpad(ir.LiteralStructType([int32, int8.as_pointer()]), pers, 'lp')
+        lp = builder.landingpad(ir.LiteralStructType([int32, int8.as_pointer()]), 'lp')
         int_typeinfo = ir.GlobalVariable(builder.function.module, int8.as_pointer(), "_ZTIi")
         int_typeinfo.global_constant = True
         lp.add_clause(ir.CatchClause(int_typeinfo))
@@ -789,7 +812,7 @@ class TestBuildInstructions(TestBase):
         builder.resume(lp)
         self.check_block(block, """\
             my_block:
-                %"lp" = landingpad {i32, i8*} personality i8 (...)* @"__gxx_personality_v0"
+                %"lp" = landingpad {i32, i8*}
                     catch i8** @"_ZTIi"
                     filter [1 x i8**] [i8** @"_ZTIi"]
                 resume {i32, i8*} %"lp"
@@ -1242,6 +1265,8 @@ class TestConstant(TestBase):
         self.assertEqual(str(c), 'i1 false')
         c = ir.Constant(int1, ir.Undefined)
         self.assertEqual(str(c), 'i1 undef')
+        c = ir.Constant(int1, None)
+        self.assertEqual(str(c), 'i1 0')
 
     def test_reals(self):
         # XXX Test NaNs and infs
@@ -1255,6 +1280,8 @@ class TestConstant(TestBase):
         self.assertEqual(str(c), 'double 0xbff8000000000000')
         c = ir.Constant(dbl, ir.Undefined)
         self.assertEqual(str(c), 'double undef')
+        c = ir.Constant(dbl, None)
+        self.assertEqual(str(c), 'double 0.0')
 
     def test_arrays(self):
         c = ir.Constant(ir.ArrayType(int32, 3), (c32(5), c32(6), c32(4)))
@@ -1263,22 +1290,85 @@ class TestConstant(TestBase):
         self.assertEqual(str(c), '[2 x i32] [i32 5, i32 undef]')
         c = ir.Constant(ir.ArrayType(int32, 2), ir.Undefined)
         self.assertEqual(str(c), '[2 x i32] undef')
-        c = ir.Constant(ir.ArrayType(int8, 3), bytearray(b"\x01\x04\xff"))
-        self.assertEqual(str(c), r'[3 x i8] c"\01\04\ff"')
+        c = ir.Constant(ir.ArrayType(int32, 2), None)
+        self.assertEqual(str(c), '[2 x i32] zeroinitializer')
+        # Raw array syntax
+        c = ir.Constant(ir.ArrayType(int8, 11), bytearray(b"foobar_123\x80"))
+        self.assertEqual(str(c), r'[11 x i8] c"foobar\5f123\80"')
+        c = ir.Constant(ir.ArrayType(int8, 4), bytearray(b"\x00\x01\x04\xff"))
+        self.assertEqual(str(c), r'[4 x i8] c"\00\01\04\ff"')
+        # Recursive instantiation of inner constants
+        c = ir.Constant(ir.ArrayType(int32, 3), (5, ir.Undefined, 6))
+        self.assertEqual(str(c), '[3 x i32] [i32 5, i32 undef, i32 6]')
+        # Invalid number of args
+        with self.assertRaises(ValueError):
+            ir.Constant(ir.ArrayType(int32, 3), (5, 6))
 
     def test_structs(self):
-        c = ir.Constant(ir.LiteralStructType((flt, int1)),
-                        (ir.Constant(ir.FloatType(), 1.5),
-                         ir.Constant(int1, True)))
+        st1 = ir.LiteralStructType((flt, int1))
+        st2 = ir.LiteralStructType((int32, st1))
+        c = ir.Constant(st1, (ir.Constant(ir.FloatType(), 1.5),
+                              ir.Constant(int1, True)))
         self.assertEqual(str(c), '{float, i1} {float 0x3ff8000000000000, i1 true}')
         c = ir.Constant.literal_struct((ir.Constant(ir.FloatType(), 1.5),
                                         ir.Constant(int1, True)))
+        self.assertEqual(c.type, st1)
         self.assertEqual(str(c), '{float, i1} {float 0x3ff8000000000000, i1 true}')
         c = ir.Constant.literal_struct((ir.Constant(ir.FloatType(), 1.5),
                                         ir.Constant(int1, ir.Undefined)))
+        self.assertEqual(c.type, st1)
         self.assertEqual(str(c), '{float, i1} {float 0x3ff8000000000000, i1 undef}')
-        c = ir.Constant(ir.LiteralStructType((flt, int1)), ir.Undefined)
+        c = ir.Constant(st1, ir.Undefined)
         self.assertEqual(str(c), '{float, i1} undef')
+        c = ir.Constant(st1, None)
+        self.assertEqual(str(c), '{float, i1} zeroinitializer')
+        # Recursive instantiation of inner constants
+        c1 = ir.Constant(st1, (1.5, True))
+        self.assertEqual(str(c1), '{float, i1} {float 0x3ff8000000000000, i1 true}')
+        c2 = ir.Constant(st2, (42, c1))
+        self.assertEqual(str(c2), '{i32, {float, i1}} '
+                                  '{i32 42, {float, i1} {float 0x3ff8000000000000, i1 true}}')
+        c3 = ir.Constant(st2, (42, (1.5, True)))
+        self.assertEqual(str(c3), str(c2))
+        # Invalid number of args
+        with self.assertRaises(ValueError):
+            ir.Constant(st2, (4, 5, 6))
+
+    def test_type_instantiaton(self):
+        """
+        Instantiating a type should create a constant.
+        """
+        c = int8(42)
+        self.assertIsInstance(c, ir.Constant)
+        self.assertEqual(str(c), 'i8 42')
+        c = int1(True)
+        self.assertIsInstance(c, ir.Constant)
+        self.assertEqual(str(c), 'i1 true')
+        # Arrays
+        at = ir.ArrayType(int32, 3)
+        c = at([c32(4), c32(5), c32(6)])
+        self.assertEqual(str(c), '[3 x i32] [i32 4, i32 5, i32 6]')
+        c = at([4, 5, 6])
+        self.assertEqual(str(c), '[3 x i32] [i32 4, i32 5, i32 6]')
+        c = at(None)
+        self.assertEqual(str(c), '[3 x i32] zeroinitializer')
+        with self.assertRaises(ValueError):
+            at([4, 5, 6, 7])
+        # Structs
+        st1 = ir.LiteralStructType((flt, int1))
+        st2 = ir.LiteralStructType((int32, st1))
+        c = st1((1.5, True))
+        self.assertEqual(str(c), '{float, i1} {float 0x3ff8000000000000, i1 true}')
+        c = st2((42, (1.5, True)))
+        self.assertEqual(str(c), '{i32, {float, i1}} '
+                                 '{i32 42, {float, i1} {float 0x3ff8000000000000, i1 true}}')
+
+    def test_repr(self):
+        """
+        Constants should have a useful repr().
+        """
+        c = int32(42)
+        self.assertEqual(repr(c), "<ir.Constant type='i32' value=42>")
 
     def test_encoding_problem(self):
         c = ir.Constant(ir.ArrayType(ir.IntType(8), 256),
@@ -1303,6 +1393,16 @@ class TestConstant(TestBase):
             'getelementptr ({float, i1}, {float, i1}* @"myconstant", i32 0, i32 1)')
         self.assertEqual(c.type, ir.PointerType(int1))
 
+    def test_bitcast(self):
+        m = self.module()
+        gv = ir.GlobalVariable(m, int32, "myconstant")
+        c = gv.bitcast(int64.as_pointer())
+        self.assertEqual(str(c), 'bitcast (i32* @"myconstant" to i64*)')
+
+    def test_inttoptr(self):
+        c = ir.Constant(int32, 0).inttoptr(int64.as_pointer())
+        self.assertEqual(str(c), 'inttoptr (i32 0 to i64*)')
+
 
 class TestTransforms(TestBase):
     def test_call_transform(self):
